bdlc = CreateFrame("frame", nil, UIParent)


defaults = {
	council_min_rank = 1,
	custom_council = {}
}
config = {
	flat = "Interface\\Buttons\\WHITE8x8",
	height = 400,
	width = 600,
	debug = false,
	version = "6.0"
}
bdlc.frames = {
	entries = {},
	tabs = {},
	rolls = {}
}

message_prefix = "BDLC";
local_player = select(1, UnitName("player"))
local loot_sessions = {
	--[itemLink] = itemUID
}
loot_council_votes = {
 -- itemLink[raidIndex] = {councilNames}
}
local versions = {}
local loot_considering = {}
local loot_want = {}
local items_waiting = {}
local player_items_waiting = {}
local sample_items = {
	GetInventoryItemLink("player", 14),
	GetInventoryItemLink("player", 1),
	GetInventoryItemLink("player", 16),
	GetInventoryItemLink("player", 15),
}
local user_considerations = {}
local loot_council = {}

function debug(msg)
	if (config.debug) then
		print(msg)
	end
end

local function skinBackdrop(frame, ...)
    frame:SetBackdrop({bgFile = config.flat, edgeFile = config.flat, edgeSize = 2})
    frame:SetBackdropColor(...)
    frame:SetBackdropBorderColor(0,0,0,1)
end
local function inLC()
	if (loot_council[local_player]) then
		return true
	else
		return false
	end
end

local function addUserConsidering(itemLink, raidIndex, iLvL, guildRank, rankIndex)
	if not inLC() then return false end
	if (not loot_sessions[itemLink]) then return false end

	local currententry = nil
	for i = 1, #bdlc.frames.tabs do
		if (bdlc.frames.tabs[i].itemLink == itemLink) then
			for e = 1, #bdlc.frames.entries[i] do
				if (not currententry and not bdlc.frames.entries[i][e].active) then
					currententry = bdlc.frames.entries[i][e]
					currententry.active = true
					currententry.itemLink = itemLink
					currententry.rankIndex = rankIndex
					currententry.wantLevel = 0
					currententry.notes = ""
					currententry.raidIndex = raidIndex
				end
			end
		end
	end
	
	if (not currententry) then return false end
	
	local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(raidIndex)
	local itemName, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemLink)
	--local guildName, guildRankName, guildRankIndex = GetGuildInfo(name)
	local class, classFileName = UnitClass(name)
	local color = RAID_CLASS_COLORS[classFileName]
	
	currententry:Show()
	currententry.name:SetText(name)
	currententry.name:SetTextColor(color.r,color.g,color.b);
	currententry.interest:SetText("considering...");
	currententry.interest:SetTextColor(.5,.5,.5);
	currententry.rank:SetText(guildRank)
	currententry.ilvl:SetText(iLvL)
	currententry.gear1:Hide()
	currententry.gear2:Hide()
	
	if (IsMasterLooter()) then
		currententry.removeUser:Show()
	end
	
	loot_considering[itemLink][raidIndex] = {itemLink, raidIndex, iLvL, guildRank}
	
	bdlc:repositionFrames()
end

local function createVoteWindow(itemLink)
	
	local name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemLink)
	lcVoteFrame:Show()
	local currenttab = nil
	for i = 1, #bdlc.frames.tabs do
		if (not currenttab and not bdlc.frames.tabs[i].active) then
			currenttab = bdlc.frames.tabs[i]
			currenttab.active = true
			currenttab.itemLink = itemLink
		end
	end
	
	-- Set Up tab and item info
	currenttab:Show()
	currenttab.icon:SetTexture(texture)
	currenttab.table.item.itemtext:SetText(link)
	currenttab.table.item.icon.tex:SetTexture(texture)
	local slotname = string.lower(string.gsub(equipSlot, "INVTYPE_", ""));
	slotname = slotname:gsub("^%l", string.upper)
	currenttab.table.item.itemdetail:SetText("ilvl: "..iLevel.."    "..subclass..", "..slotname);
	currenttab.table.item:SetScript("OnEnter", function()
		ShowUIPanel(GameTooltip)
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
		GameTooltip:SetHyperlink(link)
		GameTooltip:Show()
	end)
	currenttab.table.item:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)
	
	
	bdlc:repositionFrames()
end
local function createRollWindow(itemLink)
	local name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemLink)
	rollFrame:Show()
	
	local currentroll = nil
	for i = 1, #bdlc.frames.rolls do
		if (not currentroll and not bdlc.frames.rolls[i].active) then
			currentroll = bdlc.frames.rolls[i]
			currentroll.active = true
			currentroll.itemLink = itemLink
		end
	end
	
	currentroll:Show()
	currentroll.item.icon.tex:SetTexture(texture)
	currentroll.item.item_text:SetText(link)
	currentroll.item.item_ilvl:SetText("ilvl: "..iLevel)
	currentroll.item:SetScript("OnEnter", function()
		ShowUIPanel(GameTooltip)
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
		GameTooltip:SetHyperlink(link)
		GameTooltip:Show()
	end)
	currentroll.item:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)
	
	local _, guildRank, rankIndex = GetGuildInfo("player")
	local player_itemlvl = math.floor(select(2, GetAverageItemLevel()))
	SendAddonMessage(message_prefix, "addUserConsidering><"..itemLink.."><"..getRaidIndex("player").."><"..player_itemlvl.."><"..guildRank.."><"..rankIndex, "RAID");
	
	bdlc:repositionFrames()
end

local function endSession(itemLink)
	local currententry = nil
	local currenttab = nil
	if inLC() then 
		for i = 1, #bdlc.frames.tabs do
			if (bdlc.frames.tabs[i].itemLink == itemLink) then
				for e = 1, #bdlc.frames.entries[i] do
					if (bdlc.frames.entries[i][e].itemLink == itemLink) then
						currententry = bdlc.frames.entries[i][e]
						currententry:Hide()
						currententry.user_notes:Hide()
						currententry.active = false
						currententry.itemLink = 0
						currententry.votes.text:SetText("0")
						currententry.raidIndex = 0
						currententry.voteUser:Hide()
						currententry.rankIndex = 0
						currententry.wantLevel = 0
						currententry.notes = ""
					end
				end
				currenttab = bdlc.frames.tabs[i]
				currenttab:Hide()
				currenttab:SetAlpha(0.3)
				currenttab.table:Hide()
				currenttab.active = false
				currenttab.itemLink = 0
			end
		end
		loot_considering[itemLink] = nil
		loot_want[itemLink] = nil
	end
	local currentroll = nil
	for i = 1, #bdlc.frames.rolls do
		if (bdlc.frames.rolls[i].itemLink == itemLink) then
			currentroll = bdlc.frames.rolls[i]
			currentroll.active = false
			currentroll.itemLink = 0
			currentroll.notes = ""
			currentroll:Hide()
		end
	end
	
	loot_sessions[itemLink] = nil
	loot_council_votes[itemLink] = nil
	
	bdlc:repositionFrames()
end

local function removeUserConsidering(itemLink, raidIndex, playerName)
	
	if inLC() then 
		local currententry = nil
		for i = 1, #bdlc.frames.tabs do
			if (bdlc.frames.tabs[i].itemLink == itemLink) then
				for e = 1, #bdlc.frames.entries[i] do
					if (not currententry and bdlc.frames.entries[i][e].raidIndex == raidIndex) then
						currententry = bdlc.frames.entries[i][e]
					end
				end
			end
		end
		if (currententry) then
			currententry:Hide()
			currententry.user_notes:Hide()
			currententry.active = false
			currententry.itemLink = 0
			currententry.notes = ""
			currententry.raidIndex = 0
			currententry.rankIndex = 0
			currententry.wantLevel = 0
			currententry.voteUser:Hide()
			currententry.votes.text:SetText("0")
		end
	
		if (loot_council_votes[itemLink]) then
			loot_council_votes[itemLink][raidIndex] = nil
		end
		if (loot_considering[itemLink]) then
			loot_considering[itemLink][raidIndex] = nil
		end
		if (loot_want[itemLink]) then
			loot_want[itemLink][raidIndex] = nil
		end
	
	end
	
	SendAddonMessage(message_prefix, "removeUserRoll><"..itemLink, "WHISPER", playerName);
	
	bdlc:repositionFrames()
end

local function removeUserRoll(itemLink)
	local currentroll = nil
	for i = 1, #bdlc.frames.rolls do
		if (bdlc.frames.rolls[i].itemLink == itemLink) then
			currentroll = bdlc.frames.rolls[i]
			currentroll.active = false
			currentroll.itemLink = 0
			currentroll.notes = ""
			currentroll:Hide()
		end
	end
	
	bdlc:repositionFrames()
end

local function startSession(itemLink)

	local itemString = string.match(itemLink, "item[%-?%d:]+")
	local itemType, itemID, enchant, gem1, gem2, gem3, gem4, suffixID, uniqueID, level, upgradeId, instanceDifficultyID, numBonusIDs, bonusID1, bonusID2  = string.split(":", itemString)
	bonusID1 = bonusID1 or 0
	bonusID2 = bonusID2 or 0
	local itemUID = itemID..":"..numBonusIDs..":"..bonusID1..":"..bonusID2
	
	if (GetItemInfo(itemLink)) then
		local equipSlot = select(9, GetItemInfo(itemLink))
		local name = select(1, GetItemInfo(itemLink))
		
		if (not loot_sessions[itemLink] and (string.len(equipSlot) > 0 or strfind(name, "Vanquisher") or strfind(name, "Conqueror") or strfind(name, "Protector"))) then
			if (inLC()) then
				debug("In LC")
				createVoteWindow(itemLink)
				
				loot_council_votes[itemLink] = {}
				loot_considering[itemLink] = {}
				loot_want[itemLink] = {}
			end
			createRollWindow(itemLink)
			loot_sessions[itemLink] = itemUID
		end
	else
		items_waiting[itemID] = itemLink
	end
end
local function startMockSession()
	debug("start mock session")
	sample_items = {
		GetInventoryItemLink("player", 14),
		GetInventoryItemLink("player", 1),
		GetInventoryItemLink("player", 16),
		GetInventoryItemLink("player", 15),
	}
	for i = 1, #sample_items do
		SendAddonMessage(message_prefix, "startSession><"..sample_items[i], "RAID");
	end
end

local function buildLC()
	debug("building LC")
	local numGuildMembers, numOnline, numOnlineAndMobile = GetNumGuildMembers()
	
	for i = 1, numGuildMembers do
		local fullName, rank, rankIndex, level, class, zone, note, officernote, online, status, classFileName, achievementPoints, achievementRank, isMobile, canSoR, reputation = GetGuildRosterInfo(i)
		
		local servertest, b = string.find(fullName, "-")
		--[[local findserver = select(1, string.find(fullName, "-"))
		fullName = string.sub(fullName, 0, findserver-1)--]]
		
		if (rankIndex <= defaults.council_min_rank and servertest ~= nil) then
			SendAddonMessage(message_prefix, "addToLC><"..fullName, "RAID");
		end
	end
	--[[for k, v in pairs (bdlconfig.custom_council) do
		SendAddonMessage(message_prefix, "addToLC><"..k, "RAID");
	end--]]
end
local function addToLC(playerName)
	loot_council[playerName] = true
	--debug(playerName..' added to lc')
end

local function updateUserItem(itemLink, frame)

	local itemName, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(itemLink)
	frame:Show()
	frame.tex:SetTexture(texture)
	frame:SetScript("OnEnter", function()
		ShowUIPanel(GameTooltip)
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
		GameTooltip:SetHyperlink(link)
		GameTooltip:Show()
	end)
	frame:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)
end

local function addUserNotes(itemUID, raidIndex, notes)

	local itemLink = nil
	for link, id in pairs(loot_sessions) do
		if (itemUID == id) then
			itemLink = link
		end
	end

	debug("add user notes")
	if (not loot_sessions[itemLink]) then return false end
	if not inLC() then return false end
	
	local currententry = nil
	for i = 1, #bdlc.frames.tabs do
		if (bdlc.frames.tabs[i].itemLink == itemLink) then
			for e = 1, #bdlc.frames.entries[i] do
				if (not currententry and bdlc.frames.entries[i][e].raidIndex == raidIndex) then
					currententry = bdlc.frames.entries[i][e]
				end
			end
		end
	end
	
	if (not currententry) then return false end
	
	currententry.notes = notes
	currententry.user_notes:Show()
	
end

local function addUserWant(itemUID, raidIndex, want, itemLink1, itemLink2)
	
	local itemLink = nil
	for link, id in pairs(loot_sessions) do
		if (itemUID == id) then
			itemLink = link
		end
	end

	debug("add user want")
	if (not loot_sessions[itemLink]) then return false end
	if not inLC() then return false end
	
	local currententry = nil
	for i = 1, #bdlc.frames.tabs do
		if (bdlc.frames.tabs[i].itemLink == itemLink) then
			for e = 1, #bdlc.frames.entries[i] do
				if (not currententry and bdlc.frames.entries[i][e].raidIndex == raidIndex) then
					currententry = bdlc.frames.entries[i][e]
				end
			end
		end
	end
	
	if (not currententry) then return false end
	local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(raidIndex)
	
	local wantText = "";
	local wantColor = {};
	if (want == 1) then
		wantText = "Mainspec"
		wantColor = {.2, 1, .2};
	elseif (want == 2) then
		wantText = "Minor Up"
		wantColor = {.6, 1, .6};
	elseif (want == 3) then
		wantText = "Offspec"
		wantColor = {.8, .6, .6};
	end
	
	currententry.interest:SetText(wantText);
	currententry.interest:SetTextColor(unpack(wantColor));
	currententry.voteUser:Show()
	currententry.wantLevel = want
	
	if (GetItemInfo(itemLink1)) then
		local itemName, link1, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture1, vendorPrice = GetItemInfo(itemLink1)
		currententry.gear1:Show()
		currententry.gear1.tex:SetTexture(texture1)
		currententry.gear1:SetScript("OnEnter", function()
			ShowUIPanel(GameTooltip)
			GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
			GameTooltip:SetHyperlink(link1)
			GameTooltip:Show()
		end)
		currententry.gear1:SetScript("OnLeave", function()
			GameTooltip:Hide()
		end)
	else
		local itemID = select(2, strsplit(":", itemLink1))
		if (itemID) then
			player_items_waiting[itemID] = {itemLink1, currententry.gear1}
		end
	end
	if (itemLink2 ~= 0) then
		if (GetItemInfo(itemLink2)) then
			local itemName, link2, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture2, vendorPrice = GetItemInfo(itemLink2)
			currententry.gear2:Show()
			currententry.gear2.tex:SetTexture(texture2)
			currententry.gear2:SetScript("OnEnter", function()
				ShowUIPanel(GameTooltip)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR")
				GameTooltip:SetHyperlink(link2)
				GameTooltip:Show()
			end)
			currententry.gear2:SetScript("OnLeave", function()
				GameTooltip:Hide()
			end)
		else
			local itemID = select(2, strsplit(":", itemLink2))
			if (itemID) then
				player_items_waiting[itemID] = {itemLink2, currententry.gear2}
			end
		end
	end
	
	loot_want[itemLink][raidIndex] = {itemLink, raidIndex, want, itemLink1, itemLink2}
	
	bdlc:repositionFrames()
end

local function fetchLC()
	if (IsMasterLooter()) then
		buildLC()
	end
end

local function voteForUser(councilName, itemLink, raidIndex)
	if (not loot_sessions[itemLink]) then return false end
	if not inLC() then return false end
	
	-- make sure theres an array to represent this user in the raid
	if (not loot_council_votes[itemLink][raidIndex]) then
		loot_council_votes[itemLink][raidIndex] = {}
	end
	-- first, unset this council member on any other user for this item
	for raidIndex, v in pairs(loot_council_votes[itemLink]) do
		v[councilName] = nil
	end
	
	-- now add this council member as a vote for this raid member and this itemLink
	loot_council_votes[itemLink][raidIndex][councilName] = true
	
	-- now lets loop through open sessions, and their entries while tallying votes for said item>entry
	for itemLink, _ in pairs(loot_sessions) do
		for i = 1, #bdlc.frames.tabs do
			if (bdlc.frames.tabs[i].itemLink == itemLink) then
				for e = 1, #bdlc.frames.entries[i] do
					local currententry = bdlc.frames.entries[i][e]
					if (currententry.active and loot_council_votes[itemLink][currententry.raidIndex]) then
						local votes = 0
						for k, v in pairs(loot_council_votes[itemLink][currententry.raidIndex]) do
							votes = votes + 1
						end
						currententry.votes.text:SetText(votes)
					end
				end
			end
		end
	end
	
end

local function fetchSessions(target)
	if (IsMasterLooter()) then
	
		raidIndex = 0
		for i = 1, GetNumGroupMembers() do
			local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
			
			if (name == target) then
				raidIndex = i
			end
		end
		
		for itemLink, _ in pairs(loot_sessions) do
			SendAddonMessage(message_prefix, "removeUserConsidering><"..itemLink.."><"..raidIndex.."><"..target, "RAID");
			SendAddonMessage(message_prefix, "startSession><"..itemLink, "WHISPER", target);
		end
		
		for itemLink, indextable in pairs(loot_considering) do
			for r = 1, #indextable do
				local itemLink, raidIndex, itemLvl, guildRank = unpack(indextable[r])
				SendAddonMessage(message_prefix, "addUserConsidering><"..itemLink.."><"..raidIndex.."><"..itemLvl.."><"..guildRank, "WHISPER", target);
			end
		end
		for itemLink, indextable in pairs(loot_want) do
			for r = 1, #indextable do
				local itemLink, raidIndex, want, itemLink1, itemLink2 = unpack(indextable[r])
				SendAddonMessage(message_prefix, "addUserWant><"..itemLink.."><"..raidIndex.."><"..want.."><"..itemLink1.."><"..itemLink2, "WHISPER", target);
			end
		end
		
	end
end

local function sendVersion(version, player)
	if (not versions[version]) then
		versions[version] = {}
	end
	versions[version][player] = true
end

local function versionCheck(checker)
	local version = GetAddOnMetadata("BDLC", "Version") 
	SendAddonMessage(message_prefix, "sendVersion><"..version.."><"..local_player, "WHISPER", checker);
end

local versioncheck = CreateFrame('frame', nil, UIParent)
local function checkVersions()
	SendAddonMessage(message_prefix, "versionCheck><"..local_player, "RAID");
	
	local noaddon = {}
	for i = 1, GetNumGroupMembers() do
		local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
		noaddon[name] = true
	end
	
	versions = {}
	local total = 0
	versioncheck:SetScript("OnUpdate", function(self, elapsed)
		total = total + elapsed
		if (total > 1) then
			total = 0;
			
			for version, value in pairs(versions) do
				local printString = version..": " 
				for name, _ in pairs (value) do
					noaddon[name] = nil
					printString = printString..name..", "
				end
				print(printString)
			end
			
			if (#noaddon > 0) then
				local printString = "BDLC not installed: "
				for name, _ in pairs(noaddon) do
					printString = printString..name..", "
				end
				print(printString)
			end
			
			versions = {}
			versioncheck:SetScript("OnUpdate", function() return end)
		end
	end)
end

local function fetchVotes()
end

bdlc:RegisterEvent("ADDON_LOADED")
bdlc:SetScript("OnEvent", function(self, event, arg1, arg2, arg3)
	if (event == "ADDON_LOADED" and arg1 == "BDLC") then
		bdlc:UnregisterEvent("ADDON_LOADED")
		-------------------------------------------------------
		--- Register necessary events
		-------------------------------------------------------
		bdlc:RegisterEvent("LOOT_OPENED");
		bdlc:RegisterEvent("LOOT_SLOT_CLEARED")
		bdlc:RegisterEvent('GET_ITEM_INFO_RECEIVED')
		bdlc:RegisterEvent('CHAT_MSG_ADDON')
		bdlc:RegisterEvent('GROUP_ROSTER_UPDATE')
		bdlc:RegisterEvent('CHAT_MSG_WHISPER')
		bdlc:RegisterEvent('PARTY_LOOT_METHOD_CHANGED')
		
		--------------------------------------------------------------------------------
		-- Load configuration or set defaults
		--------------------------------------------------------------------------------
		print("|cff3399FFBig Dumb Loot Council|r loaded. /bdlc for options")
		RegisterAddonMessagePrefix(message_prefix);
		if (not defaults) then
			defaults = defaults
		end
		
		--------------------------------------------------------------------------------
		-- Set up slash commands
		--------------------------------------------------------------------------------
		SLASH_BDLC1 = "/bdlc"
		bdlc_config_toggle = false
		SlashCmdList["BDLC"] = function(origmsg, editbox)
			origmsg = strtrim(origmsg)
			local param = split(origmsg, " ")
			local msg = param[0] or origmsg;
			if (msg == "" or msg == " ") then
				print("BCLC Options:")
				print("  /bdlc test - Tests the addon (must be in raid)")
				print("  /bdlc show - Shows the vote window (if you're in the LC)")
				print("  /bdlc hide - Hides the vote window (if you're in the LC)")
				print("  /bdlc version - Check the bdlc versions that the raid is using")
				print("  /bdlc addtolc playername - Adds a player to the loot council (if you're the Masterlooter)")
			elseif (msg == "version") then
				checkVersions()
			elseif (msg == "start") then
				local s, e = string.find(origmsg, msg)
				local newmsg = strtrim(string.sub(origmsg, e+1))
				--local msg = param[0] or msg;
				if (IsInRaid() and IsMasterLooter()) then
					SendAddonMessage(message_prefix, "startSession><"..newmsg, "RAID");
				else
					print("bdlc: You must be both in a raid and in the loot council to do that");
				end
			elseif (msg == "addtolc") then
				if (IsInRaid()) then
					if (IsMasterLooter() or UnitIsGroupLeader("player")) then
						if (param[1]) then
							defaults.custom_council[param[1]] = true
							SendAddonMessage(message_prefix, "addToLC><"..param[1], "RAID");
							print("bdlc: Adding "..param[1].." to loot council.")
							SendChatMessage("bdlc: You've been added to loot council for this raid", "WHISPER", nil, param[1])
						else
							print("bdlc: Please provide a name to add to the loot council")
						end
					else
						print("bdlc: You must be the Masterlooter to do that")
					end
				else
					print("bdlc: You must be in a raid to do that")
				end
			elseif (msg == "config") then
				if (bdlc_config_toggle) then
					bdlc_config_toggle = false
					bdlcconfig:Hide()
				else
					bdlc_config_toggle = true
					bdlcconfig:Show()
					
				end
			elseif (msg == "test") then
				if (IsInRaid()) then
					startMockSession()
				else
					print("bdlc: You must be in a raid to test")
				end
			elseif (msg == "show" and inLC()) then
				lcVoteFrame:Show()
			elseif (msg == "hide" and inLC()) then
				lcVoteFrame:Hide()
			else
				print("/bdlc "..msg.." command not recognized")
			end
		end
	
		bdlc:CreateFrames()
	end
	
	if (event == "GROUP_ROSTER_UPDATE" or event == "PARTY_LOOT_METHOD_CHANGED") then
		if (IsMasterLooter()) then
			buildLC()
		else
			SendAddonMessage(message_prefix, "fetchLC", "RAID");
			--SendAddonMessage(message_prefix, "fetchSessions:"..local_player, "RAID")
		end
		bdlc:UnregisterEvent("GROUP_ROSTER_UPDATE")
	end
	
	if (event == "LOOT_OPENED" or event == "LOOT_SLOT_CLEARED") then
		if (IsMasterLooter()) then
			if (event == "LOOT_OPENED") then
				for slot = 1, GetNumLootItems() do
					local texture, item, quantity, quality, locked = GetLootSlotInfo(slot)

					if (quality and quality > 3) then
						lcVoteFrame:Show()
					
						local itemLink = GetLootSlotLink(slot)
						
						SendAddonMessage(message_prefix, "startSession><"..itemLink, "RAID");
					end
				end
			end
		end
	end
	
	--[[
	if (event == "CHAT_MSG_WHISPER") then
		if (IsMasterLooter()) then
			local msg, sender = arg1, arg2
			local s, e = string.find(msg, "!bdlc")
			if (s) then
				local itemLink = string.match(msg,"(\124c%x+\124Hitem:.-\124h\124r)")
				local s2, e2 = string.find(msg,"(\124c%x+\124Hitem:.-\124h\124r)")
				local wantlevel = strtrim(string.sub(msg, e2+2))
				local raidIndex = 0
				local findserver = select(1, string.find(sender, "-"))
				sender = string.sub(sender, 0, findserver-1)
			
				if (loot_sessions[itemLink]) then
					for r = 1, GetNumGroupMembers() do
						local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(r)
						if (name == sender) then
							raidIndex = r
						end
					end
					
					local want = 0
					if (wantlevel == "need" or wantlevel == "bis" or wantlevel == "mainspec") then
						want = 1
					elseif(wantlevel == "minor" or wantlevel == "sidegrade") then
						want = 2
					else
						want = 3
					end
				
					local itemID = select(2, strsplit(":", itemLink))
					NotifyInspect("raid"..raidIndex)
					InspectUnit("raid"..raidIndex)
					local itemLink1, itemLink2 = bdlc:fetchUserGear("raid"..raidIndex, itemLink)
					local guildRank = select(2, GetGuildInfo("raid"..raidIndex))
					local ilvl = getUnitItemLevel("raid"..raidIndex)
					local inspectwait = CreateFrame("frame", nil)
					local total = 0
					inspectwait:SetScript("OnUpdate", function(self, elapsed)
						total = total + elapsed
						if (total > 1) then
							total = 0
							inspectwait:SetScript("OnUpdate", function() return end)
							guildRank = select(2, GetGuildInfo(sender)) or ""
							itemLink1, itemLink2 = bdlc:fetchUserGear("raid"..raidIndex, itemLink)
							ilvl = getUnitItemLevel("raid"..raidIndex)
							
							print(itemLink1)
							print(itemLink2)
							
							SendAddonMessage(message_prefix, "addUserConsidering><"..itemLink.."><"..raidIndex.."><"..ilvl.."><"..guildRank, "RAID");
							SendAddonMessage(message_prefix, "addUserWant><"..itemID.."><"..raidIndex.."><1><"..itemLink1.."><"..itemLink2, "RAID");
							
							ClearInspectPlayer()
						end
					end)
					
					
				end
			end
		end
	end--]]
	
	
	if (event == "CHAT_MSG_ADDON" and arg1 == message_prefix) then
		local method, partyMaster, raidMaster = GetLootMethod()
		if (method == "master") then
			debug(event..", "..arg1..", "..arg2..", "..arg3)
			local param = split(arg2, "><")
			local action = param[0] or arg2;
			
			
			-- the numbers were made strings by the chat_msg_addon, lets find our numbers and convert them tonumbers
			for p = 0, #param do
				local test = param[p]
				if (tonumber(test)) then
					param[p] = tonumber(param[p])
				end
			end
			if (action == "startSession") then
				startSession(param[1])
			elseif (action == "fetchLC") then
				fetchLC()
			elseif (action == "sendVersion") then
				sendVersion(param[1], param[2])
			elseif (action == "versionCheck") then
				versionCheck(param[1])
			elseif (action == "addToLC") then
				addToLC(param[1])
			elseif (action == "removeUserConsidering") then
				removeUserConsidering(param[1], param[2], param[3])
			elseif (action == "addUserConsidering") then
				addUserConsidering(param[1], param[2], param[3], param[4], param[5])
			elseif (action == "addUserWant") then
				addUserWant(param[1], param[2], param[3], param[4], param[5])
			elseif (action == "addUserNotes") then
				addUserNotes(param[1], param[2], param[3])
			elseif (action == "fetchSessions") then
				fetchSessions(param[1])
			elseif (action == "voteForUser") then
				voteForUser(param[1], param[2], param[3])
			elseif (action == "removeUserRoll") then
				removeUserRoll(param[1])
			elseif (action == "endSession") then
				endSession(param[1])
			else
				
			end
		end
	end
		
	
	-- THIS IS FINISHED DONT TOUCH
	if (event == "GET_ITEM_INFO_RECEIVED") then	
		-- Queue items that are starting sessions
		for k, v in pairs(items_waiting) do
			local num1 = tonumber(arg1)
			local num2 = tonumber(k)
			if (num1 == num2) then
				startSession(v)
				items_waiting[k] = nil
			end
		end
		
		-- Queue items that are showing user's current gear
		for itemID, v in pairs(player_items_waiting) do
			local num1 = tonumber(arg1)
			local num2 = tonumber(itemID)
			if (num1 == num2) then
				updateUserItem(v[1], v[2])
				player_items_waiting[itemID] = nil
			end
		end
	end
	
end)